#pragma once

#include <atlcomtime.h>

namespace tipupdater
{

const int WM_CLOSEMESSAGE = WM_USER+2;
class TimerEventTrigger
{
public:
	TimerEventTrigger() : m_interval(3600)
	{
	}

	virtual ~TimerEventTrigger()
	{

	}

	virtual BOOL Run() = 0;
	virtual void Error() = 0;

	void			SetNextCheckTime()
	{
		m_nextCheckTime = COleDateTime::GetCurrentTime();
		m_nextCheckTime += COleDateTimeSpan(0,0,0,m_interval);
	}

	COleDateTime	GetNextCheckTime()
	{
		return m_nextCheckTime;
	}

	void			SetInterval( int interval ){ m_interval = interval; }
	int				GetInterval(){ return m_interval; }
private:
	COleDateTime	m_nextCheckTime;
	int				m_interval;
};

class TimerEventTriggerWrapper
{
public:
	TimerEventTriggerWrapper() : m_trigger(NULL)
	{

	}

	TimerEventTriggerWrapper( const TimerEventTriggerWrapper& _rhs )
	{
		m_trigger = _rhs.m_trigger;
	}

	void Remove()
	{
		if( m_trigger )
		{
			delete m_trigger;
			m_trigger = NULL;
		}
	}

	void SetNextCheckTime()
	{
		if( m_trigger )
			m_trigger->SetNextCheckTime();
	}

	COleDateTime GetNextCheckTime()
	{
		if( m_trigger )
		{
			return m_trigger->GetNextCheckTime();
		}

		COleDateTime tmpTime = COleDateTime::GetCurrentTime();
		tmpTime += COleDateTimeSpan(365,0,0,0);
		return tmpTime;
	}

	void Set( TimerEventTrigger *trigger ){ m_trigger = trigger; }
	BOOL Run()
	{
		if( m_trigger )
		{
			return m_trigger->Run();
		}

		return TRUE;
	}

	void Error()
	{
		if( m_trigger )
		{
			return m_trigger->Error();
		}
	}

private:
	TimerEventTrigger *m_trigger;
};

class RunTimerEventTrigger
{
public:

	bool DatesEqual(COleDateTime &odt1, COleDateTime &odt2)
	{
		CString str1 = odt1.Format();
		CString str2 = odt2.Format();

		return (!str1.Compare(str2));
	}

	bool DateSmallerThan(COleDateTime &odt1, 
		COleDateTime &odt2)
	{
		if (DatesEqual(odt1, odt2)) 
			return false;
		else 
			return odt1 < odt2;
	}


	bool operator()(TimerEventTriggerWrapper &arg)
	{
		COleDateTime tmpTime = COleDateTime::GetCurrentTime();
		if( DateSmallerThan(arg.GetNextCheckTime(), tmpTime))
		{
			BOOL ret = arg.Run();
			if( FALSE == ret )
				arg.Error();

			arg.SetNextCheckTime();

			return ret ? true : false;
		}

		
		return false;
	}
};

class RemoveTimerEventTrigger
{
public:
	bool operator()(TimerEventTriggerWrapper &arg)
	{
		arg.Remove();
		return true;
	}
};

class TimerEventTriggerHandler : public CNMessageHandler
{
public:
	DECLARE_WND_CLASS(_T("TimerEventTriggerHandler")) 
	// If this macro is not specified in your
	// class, ATL will generate a class name

	// [:2008-12-18] Memory Leak  :  OnCloseMessage ȣ ȵǾ Ҹڿ 
	virtual ~TimerEventTriggerHandler()
	{
		std::for_each( m_timerEventTriggers.begin(), m_timerEventTriggers.end(), RemoveTimerEventTrigger() );
	}

	BOOL Start(int interval_first, int interval_time = 0)
	{
		Create();
		::SetTimer( m_hWnd, WM_USER+1, interval_first*1000, NULL );
		m_interval_time = interval_time;
		return TRUE;
	}

	void Add(TimerEventTrigger *trigger)
	{
		TimerEventTriggerWrapper triggerWrapper;
		triggerWrapper.Set( trigger );

		m_timerEventTriggers.push_back( triggerWrapper );
	}

	LRESULT OnTimer(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		// ι° Ÿ̸ interval   ķδ  
		if (m_interval_time > 0 )
		{
			::KillTimer( m_hWnd, WM_USER+1);
			::SetTimer( m_hWnd, WM_USER+1, m_interval_time*1000, NULL );
			m_interval_time = 0;
		}
		std::for_each( m_timerEventTriggers.begin(), m_timerEventTriggers.end(), RunTimerEventTrigger() );
		return TRUE;
	}

	BEGIN_MSG_MAP(TimerEventTriggerHandler)
		MESSAGE_HANDLER(WM_TIMER,				OnTimer)
		MESSAGE_HANDLER(WM_CLOSEMESSAGE,		OnCloseMessage)
	END_MSG_MAP()

	void Close()
	{
		KillTimer( WM_USER+1 );
		PostMessage(WM_CLOSEMESSAGE);
	}

private:
	LRESULT OnCloseMessage(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		std::for_each( m_timerEventTriggers.begin(), m_timerEventTriggers.end(), RemoveTimerEventTrigger() );
		return 1;
	}

private:
	typedef std::list<TimerEventTriggerWrapper> TimerEventTriggerWrapperList;
	TimerEventTriggerWrapperList m_timerEventTriggers;
	int m_interval_time;
};

};